/******************************************************************************
  (c) Copyright 2002,2003, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile$
  Version:       $Name$ $Revision$
  Last Modified: $Date$
 *****************************************************************************/

package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.Utils;
import org.netkernel.layer1.util.Base64;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

/**
 * Accessor for data: scheme IETF RFC2397  http://www.ietf.org/rfc/rfc2397.txt
 * @author  tab
 */
public class DataSchemeAccessor extends StandardAccessorImpl
{
	public DataSchemeAccessor()
	{	this.declareThreadSafe();
		this.declareSourceRepresentation(IReadableBinaryStreamRepresentation.class);
	}
	
	
	public void onExists(INKFRequestContext aContext) throws Exception
	{	aContext.createResponseFrom(Boolean.TRUE);
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{	INKFRequestReadOnly reqIn=aContext.getThisRequest();
		String mimetype=reqIn.getArgumentValue("mimetype");
		if (mimetype==null)
		{	mimetype="text/plain";
		}
		boolean asString = mimetype.equals("text/plain") || reqIn.getRepresentationClass()==String.class;
		
		String encoding=reqIn.getArgumentValue("encoding");
		String charset=reqIn.getArgumentValue("charset");
		if (charset==null)
		{	charset="UTF-8";
		}
		String data=reqIn.getArgumentValue("data");
		
		Object result;
		if (encoding!=null)
		{	byte[] bytes=Base64.decode(data);
			if (asString)
			{	result = new String(bytes,charset);
			}
			else
			{	result = new ByteArrayRepresentation(bytes,charset);
			}
		}
		else
		{	result=Utils.decode(data);
		}
		
		INKFResponse response=aContext.createResponseFrom(result);
		response.setMimeType(mimetype);
	}

}
