package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.WrappedThrowable;
import org.netkernel.module.standard.endpoint.TransparentOverlayImpl;

public class ExceptionHandlerOverlay extends TransparentOverlayImpl
{
	public final static String EXCEPTION_HANDLER_CONFIG="res:/etc/ExceptionHandlerConfig.xml";
	
	public ExceptionHandlerOverlay() throws Exception
	{	declareThreadSafe();
		declareUnhandledExceptionsExpired(false);
	}
	
	@Override
	public void onRequest(String aID, INKFRequestContext context) throws Exception
	{	INKFRequest subReq;
		try
		{	subReq=context.getThisRequest().getIssuableClone();
			subReq.injectRequestScope(this.getDelegateSpace());
			INKFResponseReadOnly<?> resp=context.issueRequestForResponse(subReq);
			context.createResponseFrom(resp);
		}
		catch(NKFException e)
		{	
			//Source the declarative handler specification
			INKFRequest req=context.createRequest(EXCEPTION_HANDLER_CONFIG);
			req.injectRequestScope(this.getDelegateSpace());
			req.setRepresentationClass(IHDSNode.class);
			IHDSNode config=(IHDSNode)context.issueRequest(req);
			
			//Delegate Exception Processing
			String id=e.getDeepestId();
			IHDSNodeList nl=config.getNodes("config/exceptionHandler");
			INKFResponseReadOnly handledResponse=null;
			for(int i=0; i<nl.size(); i++)
			{	IHDSNode n=nl.get(i);
				String testid=(String)n.getFirstValue("id");
				if(testid.equals(id)||testid.equals("default"))
				{	req=context.createRequestToEndpoint((String)n.getFirstValue("target"));
					req.addArgumentByValue("failedRequest", context.getThisRequest());
					req.addArgumentByValue("exception", new WrappedThrowable(e));
					req.addArgumentByValue("handler", n);
					req.injectRequestScope(this.getDelegateSpace());
					handledResponse=context.issueRequestForResponse(req);
					break;
				}
			}
			if(handledResponse==null)
			{	throw e;			//No registered handler so rethrow.				
			}
			context.createResponseFrom(handledResponse);
		}
	}

}
