package org.netkernel.layer1.endpoint;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.net.JarURLConnection;
import java.net.URI;
import java.net.URL;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.urii.FileExpiry;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class JarSchemeAccessor extends StandardAccessorImpl
{
    private static final int DEFAULT_INITIAL_BUFFER_SIZE_BYTES = 1024;
	public JarSchemeAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	String jar=context.getThisRequest().getArgumentValue("jar");
		int start=jar.indexOf("file:");
		if(start < 0)
		{	throw new Exception("jar: scheme only supports local files like jar:file:/myfile.jar!/hello.txt");			
		}
		int stop=jar.indexOf("!/");
		URL url=new URL(jar);
		JarURLConnection jarConnection = (JarURLConnection)url.openConnection();
		ByteArrayOutputStream baos=new ByteArrayOutputStream(DEFAULT_INITIAL_BUFFER_SIZE_BYTES);
		Utils.pipe(jarConnection.getInputStream(), baos);
		
		INKFResponse resp=context.createResponseFrom(new ByteArrayRepresentation(baos.toByteArray()));
		String mime=jarConnection.getContentType();
		resp.setMimeType(mime);
		
		long pollPeriod = getKernel().getConfiguration().getLong("netkernel.poll",1000L);
		File f=new File(URI.create(jar.substring(start,stop)));
		resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, new FileExpiry(f,pollPeriod));
	}
}
