package org.netkernel.layer1.endpoint;

import java.util.*;

import org.netkernel.layer0.nkf.INKFExpiryFunction;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.util.GoldenThreadExpiryFunction;
import org.netkernel.layer0.util.PairList;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.ISpaceWithIdentity;

public class SimpleImportDiscoveryAccessor extends StandardAccessorImpl
{
    private GoldenThreadExpiryFunction mExpiry;
    private long mInitDelay;
    private String mType;
    
    public SimpleImportDiscoveryAccessor()
    {    this.declareThreadSafe();
    }
    
    public void postCommission(INKFRequestContext aContext) throws Exception
    {  try
    	{	mInitDelay=aContext.getKernelContext().getKernel().getConfiguration().getLong("netkernel.simpleimportdiscovery.init.delay");
    	}
    	catch(Exception e)
    	{ ; } //No property set
        mType=(String)aContext.getParamValue("type");
        if(mInitDelay > 0)
        {    mExpiry=new GoldenThreadExpiryFunction("SID delay");
            Thread t=new Thread()
            {
                public void run()
                {    try
                    {    Thread.sleep(mInitDelay);
                    } catch (Exception e) {;}
                    System.out.println("Activating Delayed SimpleDynamicImport "+mType);
                    mExpiry.invalidate();
                    mExpiry = null;
                }
            };
            t.start();
        }
    }
    
    public void onSource(INKFRequestContext aContext) throws NKFException
    {
        INKFExpiryFunction expiry=mExpiry;
        if (expiry==null)
        {
            String type=(String)this.getParameter("type");
            String uri=(String)this.getParameter("uri");
           
            Set<ISpace> superStack=new HashSet();
            IRequestScopeLevel scope=aContext.getKernelContext().getThisKernelRequest().getRequestScope();
            while (scope!=null)
            {    ISpace s=scope.getSpace();
                superStack.add(s);
                scope=scope.getParent();
            }
           
            aContext.logFormatted(INKFRequestContext.LEVEL_FINE, "LOG_DYNAMIC_IMPORT_TYPE", uri, type);
           
            INKFRequest req=aContext.createRequest("active:spaceAggregateHDS");
            req.addArgument("uri", uri);
            req.addArgumentByValue("exclusions", superStack);
            req.setRepresentationClass(IHDSNode.class);
            IHDSNode n=(IHDSNode)aContext.issueRequest(req);
            IHDSNodeList nl=n.getNodes("/spaces/space").filter(HDSPredicateFactory.hasChild("connection"));
            HDSBuilder b = new HDSBuilder();
            boolean importFound=false;
            b.pushNode("config");
            for(IHDSNode m : nl)
            {    for(IHDSNode m2: m.getNodes("connection/type"))
                {    String t = m2.getValue().toString();
                    if(type.equals(t))
                    {    String id=(String)m.getFirstValue("id");
                        String version=(String)m.getFirstValue("version");
                        aContext.logFormatted(INKFRequestContext.LEVEL_FINE, "LOG_DYNAMIC_IMPORT", id, version);
                        b.pushNode("space");
                        b.addNode("uri",id);
                        b.addNode("version",version);
                        b.popNode();
                        importFound=true;
                        break;
                    }
                }
            }
            if(!importFound)
            {    aContext.logFormatted(INKFRequestContext.LEVEL_FINER, "LOG_DYNAMIC_IMPORT_TYPE_NOT_FOUND", uri, type);
            }
            aContext.createResponseFrom(b.getRoot());
        }
        else
        {    //dummy response
            HDSBuilder b = new HDSBuilder();
            b.pushNode("config");
         aContext.createResponseFrom(b.getRoot()).setExpiry(INKFResponse.EXPIRY_FUNCTION,expiry);
        }
    }
}

