package org.netkernel.layer1.endpoint;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.IVersion;
import org.netkernel.urii.impl.Version;
import org.w3c.dom.Document;

public class WormholeAccessor extends StandardAccessorImpl
{
	public WormholeAccessor()
	{	this.declareThreadSafe();		
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IKernel kernel=aContext.getKernelContext().getKernel();
		IVersion version=null;
		if(aContext.getThisRequest().argumentExists("version"))
		{	version=new Version(aContext.getThisRequest().getArgumentValue("version"));
		}
		String spaceId=aContext.getThisRequest().getArgumentValue("space");
		ISpace targetSpace=kernel.getSpace(new SimpleIdentifierImpl(spaceId), version, version);
		if (targetSpace==null)
		{	throw aContext.createFormattedException("EX_SPACE_NOT_FOUND", "MSG_SPACE_NOT_FOUND", null, null, spaceId);
		}
		IRequestScopeLevel scope=RequestScopeLevelImpl.createOrphanedRootScopeLevel(targetSpace, aContext.getKernelContext().getRequestScope());		
		INKFRequest req;
		if(aContext.getThisRequest().argumentExists("uri"))
		{	req=aContext.createRequest(aContext.getThisRequest().getArgumentValue("uri"));
			req.setRequestScope(scope);
		}
		else if (aContext.getThisRequest().argumentExists("request"))
		{
			Document requestSpecification = aContext.source("arg:request",Document.class);
			aContext.getKernelContext().setRequestScope(scope);
			RequestBuilder rb = new RequestBuilder(requestSpecification,kernel.getLogger());
			req=rb.buildRequest(aContext, null, null);
		}
		else
		{	throw aContext.createFormattedException("EX_WORMHOLE_ARGUMENT_ERROR", "MSG_WORMHOLE_ARGUMENT_ERROR", null, null);			
		}
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		aContext.createResponseFrom(resp);
	}
}
