package org.netkernel.layer1.endpoint.hds;

import java.util.Collections;

import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSIndex;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSNodeListImpl;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class HDSFragmentor extends StandardAccessorImpl
{
	public HDSFragmentor()
	{	this.declareThreadSafe();
		this.declareSourceRepresentation(IHDSNode.class);
		this.declareSourceRepresentation(IHDSNodeList.class);
		this.declareArgument(new SourcedArgumentMetaImpl("operand",null,null,new Class[] {IHDSNodeList.class,IHDSIndex.class}));
		this.declareArgument(new SourcedArgumentMetaImpl("index",null,null,new Class[] {Object.class}));
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		Object operand=aContext.source("arg:operand");
		Class requested=aContext.getThisRequest().getRepresentationClass();
		Object result;
		if (operand instanceof IHDSNodeList)
		{	int index=aContext.source("arg:index",Integer.class);
			IHDSNode node=((IHDSNodeList)operand).get(index);
			if (IHDSNodeList.class.isAssignableFrom(requested))
			{	result=new HDSNodeListImpl(Collections.singletonList(node));
			}
			else
			{	result=node;
			}
		}
		else if (operand instanceof IHDSIndex)
		{	Object index=aContext.source("arg:index");
			IHDSNodeList nodes=((IHDSIndex)operand).get(index);
			if (IHDSNode.class.isAssignableFrom(requested))
			{	int s=nodes.size();
				if (s==1)
				{	result=nodes.getFirstNode();
				}
				else if (s==0) 
				{	throw aContext.createFormattedException("EX_HDSXPATH_NONE", null, null, null, null);
				}
				else
				{	throw aContext.createFormattedException("EX_HDSXPATH_MANY", null, null, null, null);
				}
			}
			else
			{	result=nodes;
			}
		}
		else
		{	throw new NKFException("Unsupported operand type",operand.getClass().getName());
		}
		aContext.createResponseFrom(result);
	}

}
